"""
*  \brief     demo_read_file.py
*  \details   Python script to read .dat binary file
*  \author    Mateo Haro
*  \version   1.0
*  \date      2022-08-21
*  \pre       None
*  \copyright (c) 2022 CSEM
*
*   CSEM S.A.
*   Jaquet-Droz 1
*   CH-2000 Neuchâtel
*   http://www.csem.ch
*
*
*   THIS PROGRAM IS CONFIDENTIAL AND CANNOT BE DISTRIBUTED
*   WITHOUT THE CSEM PRIOR WRITTEN AGREEMENT.
*
*   CSEM is the owner of this source code and is authorised to use, to modify
*   and to keep confidential all new modifications of this code.
*
"""

import struct
import wave

PACKET_SIZE = 4
ID_SIZE = 4
TIMESTAMP_SIZE = 8
AUDIO_SAMPLING_FREQ = 16384
AUDIO_SAMPLE_SIZE = 3
AUDIO_SIZE = AUDIO_SAMPLE_SIZE * AUDIO_SAMPLING_FREQ
BME280_SIZE = 4 * 4
MECHANICAL_SIZE = 3 * 4
"""
CAREFUL: the checksum boolean needs to be false if the data being read has been recorded before 2023-08-09 at 15:00
and the the boolean needs to be True if the data attempted to be read has been recorded after 2023-08-09 at 15:00
"""
CHECKSUM = True # checksum has been implemented from 2023-08-09 at 15:00
SAMPLE_SIZE = PACKET_SIZE + ID_SIZE + TIMESTAMP_SIZE + AUDIO_SIZE + BME280_SIZE + MECHANICAL_SIZE
if CHECKSUM:
    SAMPLE_SIZE+=1
    

def read_data(file_name, save_in_directory):
    # open the .dat file with "rb" (read binary) mode
    with open(file_name, "rb") as f:
        data = f.read()
        f.close()
    
    # read the binary data
    num_sample = int(len(data) / SAMPLE_SIZE) # should be an integer
    for j in range(num_sample):
        index = j * SAMPLE_SIZE
        packet_size = struct.unpack("<L", data[index: index + PACKET_SIZE])[0] # should be equal to SAMPLE_SIZE
        index += PACKET_SIZE
        packet_id = struct.unpack("<L", data[index: index + ID_SIZE])[0]
        index += ID_SIZE
        timestamp = struct.unpack("<Q", data[index: index + TIMESTAMP_SIZE])[0]
        index += TIMESTAMP_SIZE
        samples_raw =  data[index: index + AUDIO_SIZE]
        index += AUDIO_SIZE
        bme_values = list(struct.iter_unpack("<f", data[index: index + BME280_SIZE]))
        index += BME280_SIZE
        mechanical_values = list(struct.iter_unpack("<I", data[index: index + MECHANICAL_SIZE]))
        index += MECHANICAL_SIZE
        
        # post process and save the BME and mechanical data in a dict
        BME_data = {"temperature":bme_values[0][0], "pressure":bme_values[1][0], "altitude":bme_values[2][0], "humidity":bme_values[3][0]}
        print(BME_data)
        mechanical_data = {"wind_count":mechanical_values[0][0],"rain_count":mechanical_values[1][0],"wind_direction":mechanical_values[2][0]}
        print(mechanical_data)

        # post process the audio samples collected with I2S protocol
        samples = []
        for i in range(0, int(AUDIO_SIZE / AUDIO_SAMPLE_SIZE)): # get the number of audio samples, not the number of bytes
            value =     samples_raw[AUDIO_SAMPLE_SIZE*i      ] << 16
            value +=    samples_raw[AUDIO_SAMPLE_SIZE*i + 1  ] << 8
            value +=    samples_raw[AUDIO_SAMPLE_SIZE*i + 2  ]
            if(value>2**23 - 1):
                value = -(2**24 - value)
            samples.append(value)

        # save sound data in a .wav file
        sound_file = f"{save_in_directory}/{packet_id}_{timestamp}.wav"
        obj = wave.open(sound_file, "w")
        obj.setnchannels(1)
        obj.setsampwidth(AUDIO_SAMPLE_SIZE)
        obj.setframerate(AUDIO_SAMPLING_FREQ)
        data_as_bytes = [struct.pack("<i", sample) for sample in samples] # store it as 4 bytes in LSB format (little endian)
        for data_bytes in data_as_bytes:
            obj.writeframes(data_bytes[0:3]) # remove the fourth unused byte one ().
        obj.close()